*----------------------------------------------------------------------------
* summary_stats.do
*----------------------------------------------------------------------------
*
* Generates output presented in Table 1 and Figures 1-3
*
*----------------------------------------------------------------------------
*variable definitions
*----------------------------------------------------------------------------
*applicant/borrower characteristics
*CRAGE			Customer age
*income			Customer monthly income (in dollars)
*Own			Dummy for customer owns home
*Lwp			Dummy for customer lives with parents
*Bank			Dummy for customer has bank account
*sale			Dummy for purchase was made

*vehicle characteristics
*BCOST			Acquisition cost of car in dollars
*TCOST			Total cost of car in dollars
*Car_Age		Car age in years
*Odom			Car mileage in miles
*Lot_Age		Time car has been on lot in days
*Price			Car price in dollars
*taxfees		Sales tax and government fees

*transaction characteristics
*Mindown		Required minimum down payment (in $)
*APRUSE			Interest rate on loan
*TERM			Loan term in months
*Total_Down		Actual down payment (in $)
*Amount_Financed 	Actual loan amouunt (in $)
*Monthly_Payment 	Actual monthly payment (in $)

*loan outcomes
*censored		Dummy if full loan term not observed in our sample
*default		Dummy if customer defaulted
*fracpaid		Fraction of loan payments made before default
*loanpayments		Present value of loan payments
*isrecovery		Dummy if car was recovered
*pvrecovery		Present value of recovery

gen grossrevenue = Total_Down + loanpayments + pvrecovery
gen netrevenue  = grossrevenue - cost - taxfees

*----------------------------------------------------------------------------
* Table 1: compute summary statistics
*----------------------------------------------------------------------------

*set vectors of characteristics
global buy_char="CRAGE income Own Lwp Bank"
global car_char="BCOST Cost Car_Age Odom Lot_Age Price"
global trans_char="Mindown APRUSE TERM Total_Down Amount_Financed Monthly_Payment"
global outcome_char="default fracpaid loanpayments isrecovery pvrecovery grossrevenue netrevenue"


*generate variables for presenting output
gsort -sale censored
gen sort=_n
gen name=""
gen obs=.
gen avg=.
gen std=.
gen p05=.
gen p95=.

*----------------------------------------------------------------------------
* Applicant characteristics (computed for all applicants)
*----------------------------------------------------------------------------
local i=1
foreach var of varlist $buy_char sale Mindown {

	*variable name
	quietly replace name="`var'_app" if sort==`i'
	
	*number of observations
	quietly egen temp=count(`var')
	quietly replace obs=temp if sort==`i'	
	drop temp
	
	*mean
	quietly egen temp=mean(`var')
	quietly replace avg=temp if sort==`i'
	drop temp

	*standard deviation
	quietly egen temp=sd(`var')
	quietly replace std=temp if sort==`i'
	drop temp

	*5th percentile
	quietly egen temp=pctile(`var'), p(5)
	quietly replace p05=temp if sort==`i'
	drop temp

	*95th percentile
	quietly egen temp=pctile(`var'), p(95)
	quietly replace p95=temp if sort==`i'
	drop temp

	local i=`i'+1
	}

list name obs avg std p05 p95 if _n<`i', clean
summ $buy_char sale Mindown
*----------------------------------------------------------------------------
* Buyer/car/transaction characteristics (computed for all sales)
*----------------------------------------------------------------------------
local i=1
foreach var of varlist $buy_char $car_char $trans_char {

	*variable name
	quietly replace name="`var'" if sort==`i'
	
	*number of observations
	quietly egen temp=count(`var') if sale
	quietly replace obs=temp if sort==`i'	
	drop temp
	
	*mean
	quietly egen temp=mean(`var') if sale
	quietly replace avg=temp if sort==`i'
	drop temp

	*standard deviation
	quietly egen temp=sd(`var') if sale
	quietly replace std=temp if sort==`i'
	drop temp

	*5th percentile
	quietly egen temp=pctile(`var') if sale, p(5)
	quietly replace p05=temp if sort==`i'
	drop temp

	*95th percentile
	quietly egen temp=pctile(`var') if sale, p(95)
	quietly replace p95=temp if sort==`i'
	drop temp

	local i=`i'+1
	}

list name obs avg std p05 p95 if _n<`i', clean
summ $buy_char $car_char $trans_char if sale==1
*----------------------------------------------------------------------------
* loan outcomes (computed only for uncensored sales)
*----------------------------------------------------------------------------

local i=1
foreach var of varlist $outcome_char {

	*variable name
	quietly replace name="`var'" if sort==`i'
	
	*number of observations
	quietly egen temp=count(`var') if sale & ~censored
	quietly replace obs=temp if sort==`i'	
	drop temp
	
	*mean
	quietly egen temp=mean(`var') if sale & ~censored
	quietly replace avg=temp if sort==`i'
	drop temp

	*standard deviation
	quietly egen temp=sd(`var') if sale & ~censored
	quietly replace std=temp if sort==`i'
	drop temp

	*5th percentile
	quietly egen temp=pctile(`var') if sale & ~censored, p(5)
	quietly replace p05=temp if sort==`i'
	drop temp

	*95th percentile
	quietly egen temp=pctile(`var') if sale & ~censored, p(95)
	quietly replace p95=temp if sort==`i'
	drop temp

	local i=`i'+1
	}

list name obs avg std p05 p95 if _n<`i', clean
summ $outcome_char if sale & ~censored
summ isrecovery if sale & default
summ pvrecovery if sale & default & isrecovery, d

*----------------------------------------------------------------------------
* Figure 1: Rate of Return Histogram
*----------------------------------------------------------------------------

*hist margin if sale & ~censored 
*hist margin if sale & ~censored & default
*hist margin if sale & ~censored & ~default

gen figure1_gross = Total_Down + loanpayments + pvrecovery if sale
gen figure1_cost = Cost + taxfees if sale
gen figure1_margin=(figure1_gross - figure1_cost)/figure1_cost if sale

forv i=1/60 {
	local low =-2+0.1*(`i'-1)
	local high=-2+0.1*`i'
	quietly gen temp`i'=1 if inrange(figure1_margin,`low',`high') & sale & ~censored & default
}
summ temp*, sep(60)
drop temp*
	
forv i=1/60 {
	local low =-2+0.1*(`i'-1)
	local high=-2+0.1*`i'
	quietly gen temp`i'=1 if inrange(figure1_margin,`low',`high') & sale & ~censored & ~default
}
summ temp*, sep(60)
drop temp*


*----------------------------------------------------------------------------
* Figure 2: Probablity of payment by risk type and down payment
*----------------------------------------------------------------------------
gen put_min = Total_Down <= Mindown
gen put_extra = Total_Down > Mindown

*all sales
table gnum if sale & ~censored, contents(mean default)

*put min down / extra down
table gnum put_min if sale & ~censored, contents(mean default)

*----------------------------------------------------------------------------
* Figure 3: Minimum down payment over time (raw data only)
*----------------------------------------------------------------------------

